@@ -263,6 +263,8 @@ bool TrapManager::is_destructed = false;
 void find_me() {
 }
 
+static std::mutex virtual_protect_mutex;
+
 void trap_threads(uint8_t* from, uint8_t* to, size_t len, const std::function<void()>& run_fn) {
     MEMORY_BASIC_INFORMATION find_me_mbi{};
     MEMORY_BASIC_INFORMATION from_mbi{};
@@ -298,6 +300,9 @@ void trap_threads(uint8_t* from, uint8_t* to, size_t len, const std::function<vo
         TrapManager::instance->add_trap(from, to, len);
     }
 
+    // Make sure we aren't working on a different address in the same memory page on a different thread.
+    std::scoped_lock vp_lock{virtual_protect_mutex};
+
     DWORD from_protect;
     DWORD to_protect;
 
